package com.dzeio.charts.axis

import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.Rect
import androidx.annotation.ColorInt
import com.dzeio.charts.ChartView
import com.dzeio.charts.utils.drawDottedLine

class YAxis<T>(
    private val chartView: ChartView
) {
    var max: T? = null
    var min: T? = null

    /**
     * Number of labels displayed on the sidebar
     */
    var labelCount: Int = 4

    @ColorInt
    var color = Color.parseColor("#FC496D")

    var textPaint: Paint = Paint().also {
        it.isAntiAlias = true
        it.color = color
        it.textSize = 30f
        it.textAlign = Paint.Align.RIGHT
    }

    var linePaint = Paint().apply {
        isAntiAlias = true
    }

    var legendEnabled = true

    private val rect: Rect = Rect()

    fun getWidth(): Int {
        var maxWidth = 0
        val max = chartView.getXMax(true)
        val vIncrement = max / labelCount
        for (i in 0 until labelCount) {
            val text = onValueFormat(vIncrement * (labelCount - i), true)
            textPaint.getTextBounds(text, 0, text.length, rect)
            if (rect.width() > maxWidth) maxWidth = rect.width()
        }
        return maxWidth + chartView.padding.toInt()
    }

    /**
     * Function to display the YAxis sidebar on the right
     *
     * it migh also display content over the Graph
     */
    fun display(canvas: Canvas, width: Int, height: Int) {
        val max = chartView.getXMax(true)
        val increment = (height - chartView.padding * 2) / labelCount
        val vIncrement = max / labelCount
        for (i in 0 until labelCount) {
            val text = onValueFormat(vIncrement * (labelCount - i), true)
            textPaint.getTextBounds(text, 0, text.length, rect)
            val posY = increment * i.toFloat()
            canvas.drawDottedLine(0f, posY + chartView.padding, width - rect.width().toFloat() - chartView.padding, posY, 40f, linePaint)
            canvas.drawText(text,
                width.toFloat() - chartView.padding, posY + chartView.padding, textPaint)
//            canvas.drawDottedLine(0f, posY, measuredWidth.toFloat(), posY, 10f, linePaint)
        }
    }

    var onValueFormat: (value: Float, shortVersion: Boolean) -> String = { it, _ -> it.toString() }

}
